/******************************************************************************

This is a part of the xHotspot Source code.

Copyright (C) 2007 Sascha Liebsch

This program is free software; you can redistribute it and/or
modify it under the terms of the GNU General Public License
as published by the Free Software Foundation; either version 2
of the License, or (at your option) any later version.

This program is distributed in the hope that it will be useful,
but WITHOUT ANY WARRANTY; without even the implied warranty of
MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the
GNU General Public License for more details.

You should have received a copy of the GNU General Public License
along with this program; if not, write to the Free Software
Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.

******************************************************************************/

#include "Hotspot.h"

#include <lsapi/lsapi.h>

#include "HotspotManager.h"


Hotspot::Hotspot(int const &id, HWND hwnd)
: active_(false)
, id_(id)
, hwnd_(hwnd)
{}

/*virtual*/
Hotspot::~Hotspot()
{}

MouseInOut const Hotspot::Check()
{
	//LSLogPrintf(4, "xHotspot", "%d check", id_);
	StopTimer();

	bool curr_inside = IsInside();
	MouseInOut result;

	if(!(curr_inside ^ last_inside_))
	{
		//LSLogPrintf(4, "xHotspot", "%d action none", id_);
		result = None;
	}
	else if(curr_inside && !last_inside_)
	{
		result = In;
		ExecuteAction(action_enter_);
		//LSLogPrintf(4, "xHotspot", "%d action in", id_);
	}
	else
	{
		result = Out;
		ExecuteAction(action_leave_);
		//LSLogPrintf(4, "xHotspot", "%d action out", id_);
	}

	last_inside_ = curr_inside;
	StartTimer();
	return result;
}

bool const Hotspot::IsActive() const
{
	return active_;
}

void Hotspot::SetActive(bool const state)
{
	//LSLogPrintf(4, "xHotspot", "%d set active: %d", id_, state);
	active_ = state;
	if (active_)
	{
		// set last_inside to current inside initially to avoid false alarms
		last_inside_ = IsInside();

		Check();
	}
	else
	{
		StopTimer();
	}
}

void Hotspot::Toggle()
{
	SetActive(!active_);
}

void Hotspot::SetInterval(int const &int_ms)
{
	//LSLogPrintf(4, "xHotspot", "%d set interval =%d", id_, interval_);
	interval_ = int_ms;
}

void Hotspot::SetActionEnter(std::string const &action)
{
	action_enter_ = action;
}

void Hotspot::SetActionLeave(std::string const &action)
{
	action_leave_ = action;
}

int const Hotspot::GetId() const
{
	return id_;
}

bool const Hotspot::IsInside()
{
	Point mouse_pos;
	GetMousePosition(mouse_pos);

	return IsInside(mouse_pos);
}

void Hotspot::ExecuteAction(std::string const &action)
{
	//LSLogPrintf(4, "xHotspot", "%d execute action: %s", id_, action.c_str());
	//LSExecuteEx(0, 0, action.c_str(), 0, 0, SW_SHOWNORMAL);
	LSExecute(0, action.c_str(), SW_SHOWNORMAL);
}

void Hotspot::GetMousePosition(Point &point)
{
	POINT m;
	GetCursorPos(&m);
	point.x = m.x;
	point.y = m.y;
}

void Hotspot::StartTimer()
{
	//LSLogPrintf(4, "xHotspot", "%d start timer", id_);
	if(interval_ != 0)
	{
		SetTimer(hwnd_, id_, interval_, 0);
	}
}

void Hotspot::StopTimer()
{
	KillTimer(hwnd_, id_);
}
